@@ -5,18 +5,22 @@ use warnings;
 
 package Scrappy;
 BEGIN {
-  $Scrappy::VERSION = '0.593';
+  $Scrappy::VERSION = '0.6';
 }
+
 use FindBin;
-use WWW::Mechanize::Pluggable;
+use WWW::Mechanize;
+use Web::Scraper;
 use File::ShareDir ':ALL';
 use File::Slurp;
 use YAML::Syck;
 use Array::Unique;
 use Try::Tiny;
+use URI;
+use URI::QueryParam;
 
 our $class_History              = [];
-our $class_Instance             = undef;
+our $class_Instance             = {};
     $YAML::Syck::ImplicitTyping = 1;
     
 BEGIN {
@@ -64,6 +68,7 @@ BEGIN {
         cursor
         denied
         history
+        reinit
     );
     %EXPORT_TAGS = ( syntax => [ @EXPORT_OK ] );
 }
@@ -71,33 +76,37 @@ BEGIN {
 
 
 sub init {
-    $class_Instance = WWW::Mechanize::Pluggable->new(@_);
-    die 'Could not create a scraper application instance, please make sure you ' .
-        'have install Scrappy and its prerequesites properly.'
-        unless defined $class_Instance;
+    bless $class_Instance, 'Scrappy';
     
-    $class_Instance->{Scrappy}       = { stash => {} };
+    $class_Instance->{Prop}          = { stash => {} };
+    $class_Instance->{Mech}          = WWW::Mechanize->new(@_);
     $class_Instance->{Mech}->{pause} = 0;
+    
     return $class_Instance;
 }
 
 
-sub reset {
+sub reinit {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    $class_Instance = {};
     return init(@_);
 }
 
 
 sub self {
-    die 'No scraper application instance found, please use the `init` method ' .
-        'before calling any other functions from your package or script.'
-        unless defined $class_Instance;
+    # if init() has never been called, try if once
+    init() unless keys %{$class_Instance};
+    die 'Could not create a scraper application instance, please make sure you ' .
+        'have installed Scrappy and its prerequesites properly.'
+        unless keys %{$class_Instance};
     return $class_Instance;
 }
 
 
 sub user_agent {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     my ($requested_user_agent) = shift;
-    self->add_header("User-Agent" => $requested_user_agent)
+    self->{Mech}->add_header("User-Agent" => $requested_user_agent)
         if defined $requested_user_agent;
     return $requested_user_agent ?
         $requested_user_agent : self->{Mech}->{headers}->{'User-Agent'};
@@ -105,13 +114,18 @@ sub user_agent {
 
 
 sub var {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     my ($key, $value) = @_;
     if (@_ == 2) {
+        # stash variable nesting
+        # ** depreciated **
+        # ** not recommended **
+        # var 'user/profile/name' => 'Mr. Foobar';
         if ($key =~ /\//) {
             $key =~ s/\/+/\//g;
             $key =~ s/(^\/)|(\/$)//g;
             my @keys = split /\//, $key;
-            my $var  = self->{Scrappy}->{stash};
+            my $var  = self->{Prop}->{stash};
             for (my $i = 0; $i < @keys; $i++) {
                 $var->{$keys[$i]} = $value
                     if ($i+1) == @keys;
@@ -122,15 +136,19 @@ sub var {
             return $value;
         }
         else {
-            self->{Scrappy}->{stash}->{$key} = $value if (@_ == 2);
-            return self->{Scrappy}->{stash}->{$key};
+            self->{Prop}->{stash}->{$key} = $value if (@_ == 2);
+            return self->{Prop}->{stash}->{$key};
         }
     }
-    return self->{Scrappy}->{stash};
+    elsif (@_ == 1) {
+        return self->{Prop}->{stash}->{$_[0]};
+    }
+    return self->{Prop}->{stash};
 }
 
 
 sub random_ua {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     my ($browser, $os) = @_;
        $browser = 'any' unless $browser;
        $browser = 'explorer' if
@@ -195,26 +213,32 @@ sub random_ua {
 
 
 sub form {
-    my $response = self->submit_form(@_);
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    my $response = self->{Mech}->submit_form(@_);
     sleep pause();
     return $response;
 }
 
 
 sub get {
-    my $request = self->get(@_);
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    my $url = URI->new(@_);
+    my $request = self->{Mech}->get($url);
     push @{$class_History}, @_;
     self->{Mech}->{cookie_jar}->scan(\&_cookies_to_session);
+    self->{Prop}->{params} = {};
+    self->{Prop}->{params} = +{ map { ($_ => $url->query_param($_) ) } $url->query_param };
     sleep pause();
-    return $request;
+    return self;
 }
 
 
 sub post {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     my ($url, $params) = @_;
     if ($url && ref($params) eq "HASH") {
         my $request =
-        self->post(
+        self->{Mech}->post(
             $url,
             'Content-Type' => 'application/x-www-form-urlencoded',
             'Content'      => $params
@@ -224,7 +248,7 @@ sub post {
         return $request;
     }
     else {
-        my $request = self->post(@_);
+        my $request = self->{Mech}->post(@_);
         self->{Mech}->{cookie_jar}->scan(\&_cookies_to_session);
         sleep pause();
         return $request;
@@ -232,58 +256,46 @@ sub post {
 }
 
 
-sub grab {
-    my ($selector, $mapping) = @_;
-    if ($mapping) {
-        if ("HASH" eq ref $mapping) {
-            my $temp = self->scrape( $selector, "data[]", $mapping );
-            return element($temp->{data});
-        }
-        else {
-            if (":all" eq lc $mapping) {
-                my $temp = self->scrape( $selector, "data[]", tattr() );
-                return element($temp->{data});
-            }
-            my $temp = self->scrape( $selector, "data[]", { selected => $mapping } );
-            return $temp->{data}[0]->{selected};
-        }
-    }
-    else {
-        my $temp = self->scrape( $selector, "data[]", { everything => 'TEXT' } );
-        return $temp->{data}[0]->{everything};
-    }
+sub param {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    my $key = shift;
+    return $key ? self->{Prop}->{params}->{$key} : self->{Prop}->{params};
 }
 
 
-sub zoom {
-    my ($html, $selector, $mapping) = @_;
+sub grab {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    my ($selector, $mapping, $html) = @_;
     
-    die "The zoom function needs html and a selector at least to function properly"
+    die "The grab function needs html and a selector at least to function properly"
         unless @_ >= 2;
-        
+    
+    $html ||= html();
+    
     if ($mapping) {
         if ("HASH" eq ref $mapping) {
             
+            my $scraper =
+                scraper {
+                    process
+                        ($selector, "data[]", $mapping) };
+            my $temp = $scraper->scrape( $html );
+            return element($temp->{data});
+        }
+        else {
+            
             if (":all" eq lc $mapping) {
                  my $scraper =
-                    WWW::Mechanize::Plugin::Web::Scraper::scraper {
-                        WWW::Mechanize::Plugin::Web::Scraper::process
+                    scraper {
+                        process
                             ($selector, "data[]", tattr()) };
                 my $temp = $scraper->scrape( $html );
                 return element($temp->{data});
             }
             
             my $scraper =
-                WWW::Mechanize::Plugin::Web::Scraper::scraper {
-                    WWW::Mechanize::Plugin::Web::Scraper::process
-                        ($selector, "data[]", $mapping) };
-            my $temp = $scraper->scrape( $html );
-            return element($temp->{data});
-        }
-        else {
-            my $scraper =
-                WWW::Mechanize::Plugin::Web::Scraper::scraper {
-                    WWW::Mechanize::Plugin::Web::Scraper::process
+                scraper {
+                    process
                         ($selector, "data[]", { selected => $mapping }) };
             my $temp = $scraper->scrape( $html );
             return $temp->{data}[0]->{selected};
@@ -291,8 +303,8 @@ sub zoom {
     }
     else {
         my $scraper =
-                WWW::Mechanize::Plugin::Web::Scraper::scraper {
-                    WWW::Mechanize::Plugin::Web::Scraper::process
+                scraper {
+                    process
                         ($selector, "data[]", { everything => 'TEXT' }) };
         my $temp = $scraper->scrape( $html );
         return $temp->{data}[0]->{everything};
@@ -301,90 +313,106 @@ sub zoom {
 
 
 sub loaded {
-    return self->success;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->success;
 }
 
 
 sub status {
-    return self->status;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->status;
 }
 
 
 sub reload {
-    my $response = self->reload;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    my $response = self->{Mech}->reload;
     sleep pause();
     return $response;
 }
 
 
 sub back {
-    my $response = self->back;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    my $response = self->{Mech}->back;
     sleep pause();
     return $response;
 }
 
 
 sub page {
-    return self->uri;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->uri;
 }
 
 
 sub response {
-    return self->response;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->response;
 }
 
 
 sub content_type {
-    return self->content_type;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->content_type;
 }
 
 
 sub domain {
-    return self->base;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->base;
 }
 
 
 sub ishtml {
-    return self->is_html;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->is_html;
 }
 
 
 sub title {
-    return self->title;
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->title;
 }
 
 
 sub text {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     return data( format => 'text');
 }
 
 
 sub html {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     return data(@_);
 }
 
 
 sub data {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     if ($_[0]) {
         unless ($_[1]) {
-            self->update_html($_[0]);
+            self->{Mech}->update_html($_[0]);
         }
     }
-    return self->content(@_);
+    return self->{Mech}->content(@_);
 }
 
 
 sub www {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     return self(@_);
 }
 
 
 sub store {
-    return self->save_content(@_);
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return self->{Mech}->save_content(@_);
 }
 
 
 sub download {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     my ($uri, $dir, $file) = @_;
     $dir =~ s/[\\\/]+$//;
      if (@_ == 3) {
@@ -411,6 +439,7 @@ sub download {
 
 
 sub list {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     #die 'The argument passed to the list method must be an arrayref'
     #    if ref($_[0]) ne "ARRAY";
     return ref($_[0]) ne "ARRAY" ? () : @{$_[0]};
@@ -418,18 +447,21 @@ sub list {
 
 
 sub fst {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     my @array = list @_;
     return shift @array;
 }
 
 
 sub lst {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     my @array = list @_;
     return pop @array;
 }
 
 
 sub session {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     if (@_ == 2) {
         my ($key, $value) = @_;
         if ($key eq "_file" && defined $value) {
@@ -482,11 +514,13 @@ sub session {
 
 
 sub config {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     return session @_;
 }
 
 
 sub cookies {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     self->{Mech}->{cookie_jar} = undef if $_[0] eq '_undef';
     return self->{Mech}->{cookie_jar};
 }
@@ -528,13 +562,15 @@ sub _cookies_to_session {
 
 
 sub proxy {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     my $proxy    = pop @_;
     my @protocol = @_;
-    return self->proxy([@protocol], $proxy);
+    return self->{Mech}->proxy([@protocol], $proxy);
 }
 
 
 sub pause {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     if ($_[0]) {
         if ($_[1]) {
             my @range = (($_[0] < $_[1] ? $_[0] : 0)..$_[1]);
@@ -558,6 +594,24 @@ sub pause {
     }
 }
 
+
+sub history {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    return @{$class_History};
+}
+
+
+sub denied {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    my ($last) = reverse history;
+    return 1 if (page ne $last);
+}
+
+
+sub new {
+    return init;
+}
+
 our @_queue = ();
 tie @_queue, 'Array::Unique';
 our $_cursor = 0;
@@ -570,11 +624,111 @@ sub cursor {
 
 
 sub queue {
+    my $self    = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
     return @_ ? push @_queue, @_ : @_queue;
 }
 
 
+sub match {
+    my $self    = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    my $pattern = shift;
+    my $url     = shift || page; $url = URI->new($url);
+    my $options = shift || {};
+    
+    die "route can't be defined without a valid URL pattern"
+        unless $pattern;
+    
+    my $route = self->{Prop}->{patterns}->{$pattern};
+    
+    # does route definition already exist?
+    unless (keys %{$route}) {
+            
+        $route = +{ on_match  => $options->{on_match} };
+        
+        # define options
+        if ( my $host = $options->{host} ) {
+            $route->{host} = $host;
+            $route->{host_re} = ref $host ? $host : qr(^\Q$host\E$);
+        }
+    
+        $route->{pattern} = $pattern;
+    
+        # compile pattern
+        my @capture;
+        $route->{pattern_re} = do {
+            if ( ref $pattern ) {
+                $route->{_regexp_capture} = 1;
+                $pattern;
+            }
+            else {
+                $pattern =~ s!
+                    \{((?:\{[0-9,]+\}|[^{}]+)+)\} | # /blog/{year:\d{4}}
+                    :([A-Za-z0-9_]+)              | # /blog/:year
+                    (\*)                          | # /blog/*/*
+                    ([^{:*]+)                       # normal string
+                !
+                    if ($1) {
+                        my ($name, $pattern) = split /:/, $1, 2;
+                        push @capture, $name;
+                        $pattern ? "($pattern)" : "([^/]+)";
+                    } elsif ($2) {
+                        push @capture, $2;
+                        "([^/]+)";
+                    } elsif ($3) {
+                        push @capture, '__splat__';
+                        "(.+)";
+                    } else {
+                        quotemeta($4);
+                    }
+                !gex;
+                qr{^$pattern$};
+            }
+        };
+        $route->{capture} = \@capture;    
+        self->{Prop}->{patterns} = +{ $route->{pattern} => $route };
+    }
+    
+    # match
+    if ( $route->{host_re} ) {
+        unless ( $url->host =~ $route->{host_re} ) {
+            return undef;
+        }
+    }
+    
+    if ( my @captured = ( $url->path =~ $route->{pattern_re} ) ) {
+        my %args;
+        my @splat;
+        if ( $route->{_regexp_capture} ) {
+            push @splat, @captured;
+        }
+        else {
+            for my $i ( 0 .. @{ $route->{capture} } - 1 ) {
+                if ( $route->{capture}->[$i] eq '__splat__' ) {
+                    push @splat, $captured[$i];
+                }
+                else {
+                    $args{ $route->{capture}->[$i] } = $captured[$i];
+                }
+            }
+        }
+        my $match =
+          +{ ( label => $route->{label} ), %args, ( @splat ? ( splat => \@splat ) : () ) };
+        if ( $route->{on_match} ) {
+            my $ret = $route->{on_match}->( self, $match );
+            return undef unless $ret;
+        }
+        self->{Prop}->{params} = +{ %args };
+        self->{Prop}->{params}->{splat} = \@splat if @splat;
+        return $match;
+    }
+    
+    return undef;
+}
+
+
 sub crawl {
+    my $self = 'Scrappy' eq ref $_[0] ? shift @_ : undef;
+    
     my @array = @_;
     
     my $actions = pop @array;
@@ -608,13 +762,39 @@ sub crawl {
     
     # process actions
     if ("hash" eq lc ref $actions) {
-        while (my($selector, $function) = each(%{$actions})) {
+        # while (my($selector, $function) = each(%{$actions}))
+        foreach my $action (keys %{$actions}) {
+            my ($selector, $function) = ($action, $actions->{$action});
+            
+            # page constraint condition
+            if ("code" ne lc ref $function) {
+                my $route = $selector;
+                $selector = (keys %{$actions->{$selector}})[0];
+                $function = $action = $actions->{$route}->{$selector};                
+                if (match $route) {
+                    my $findings = grab $selector, ':all';
+                    if ("array" eq lc ref $findings) {
+                        foreach (@{$findings}) {
+                            $function->($_);
+                        }
+                    }
+                    else {
+                        $function->($findings);
+                    }
+                }
+                goto nextPage;
+            }
+            
+            # standard no page constraint condition
             my $findings = grab $selector, ':all';
             if ("array" eq lc ref $findings) {
                 foreach (@{$findings}) {
                     $function->($_);
                 }
             }
+            else {
+                $function->($findings);
+            }
         }
     }
     
@@ -622,6 +802,8 @@ sub crawl {
     goto doPage if $_queue[++$_cursor];
 }
 
+## UTILITIES (Not OO nor DSL, Internal Only)
+
 sub element {
     my $object = shift;
        $object = [$object] unless "ARRAY" eq ref $object;
@@ -769,18 +951,7 @@ sub tattr {
     # need xml and json support maybe?
 }
 
-
-sub history {
-    return @{$class_History};
-}
-
-
-sub denied {
-    my ($last) = reverse history;
-    return 1 if (page ne $last);
-}
-
-init;
+1;
 __END__
 =pod
 
@@ -790,94 +961,81 @@ Scrappy - All Powerful Web Harvester, Spider, Scraper fully automated
 
 =head1 VERSION
 
-version 0.593
+version 0.6
 
 =head1 SYNOPSIS
 
-Crawl an entire website or more with three lines of code.
+Scrappy does it all, any way you like. Object-Oriented or DSL (Domain-Specific).
+Lets look at a simple scraper in OO context.
 
     #!/usr/bin/perl
-    use Scrappy qw/:syntax/;
-        
-    crawl 'http://somewebsite.com' {
-        'a' => sub { queue shift->href },
-        '/*' => {
-            # do something
+    use Scrappy;
+
+    my $spidy = Scrappy->new;
+    
+    $spidy->crawl('http://search.cpan.org/recent', {
+        '#cpansearch li a' => sub {
+            print shift->text, "\n";
         }
-    };
+    });
 
-Spider, Scrape or Harvest data from websites like never before, with ease.
+Now lets run the same operation again in DSL context.
 
     #!/usr/bin/perl
     use Scrappy qw/:syntax/;
-        
-    user_agent random_ua;
     
-    get 'http://search.cpan.org/recent';
-    
-    if (loaded) {
-        var date    => grab '.datecell b';
-        var modules => grab '#cpansearch li a', { name => 'TEXT', link => '@href' };
-    }
-    
-    print $_->{name}, "\n" for list var->{modules};
-
-Trace page fetches during crawling by using the 'Scrappy_Trace' environment variable, e.g. ...
-
-    $ENV{Scrappy_Trace} = 1;
-    crawl 'http://somewebsite.com' {
-        'a' => sub { queue shift->href },
-        '/*' => {
-            # do something
+    crawl 'http://search.cpan.org/recent', {
+        '#cpansearch li a' => sub {
+            print shift->text, "\n";
         }
     };
 
 =head1 DESCRIPTION
 
 Scrappy is an easy (and hopefully fun) way of scraping, spidering, and/or
-harvesting information from web pages. Internally Scrappy uses the awesome
-Web::Scraper and WWW::Mechanize modules so as such Scrappy imports its
-awesomeness. Scrappy is inspired by the fun and easy-to-use Dancer API. Beyond
-being a pretty API for WWW::Mechanize::Plugin::Web::Scraper, Scrappy also has its
-own featuer-set which makes web scraping easier and more enjoyable.
+harvesting information from web pages, web services, and more. Scrappy is a
+feature rich, flexible, intelligent web automation tool.
 
 Scrappy (pronounced Scrap+Pee) == 'Scraper Happy' or 'Happy Scraper'; If you
 like you may call it Scrapy (pronounced Scrape+Pee) although Python has a web
 scraping framework by that name and this module is not a port of that one.
 
+Scrappy is approaching version 1.0, taking on critical mass :}
+
 =head1 METHODS
 
 =head2 init
 
-Builds the scraper application instance. This function should be called before
-issuing any other commands as this function creates the application instance all
-other functions will use. This function returns the current scraper application
-instance.
+Builds the scraper application instance. This function is called automatically
+in DSL context and is otherwise irrelevent. This function creates the application
+instance all other functions will use in DSL context. This function returns the
+current scraper application instance.
 
     my $scraper = init;
 
-=head2 reset
+=head2 reinit
 
-The reset method is an alias to the init method. This function should be called before
-issuing any other commands as this function creates the application instance all
-other functions will use. This function returns the current scraper application
-instance.
+The reinit method is an alias to the init method. This function should be called
+in DSL context when a new scraper application instance in desired. This function
+will returns the new scraper application instance. Obviously in OO context, one
+would simply use Scrappy->new to create a new instance.
+
+    my $new = reinit;
 
 =head2 self
 
 This method returns the current scraper application instance which can also be
-found in the package class variable $class_Instance.
+found in the package class variable $class_Instance. This method has no pratical
+purpose in OO context and is made available to return the current scraper application
+instance in DSL context only.
 
-    init;
-    get $requested_url;
-    my $scraper = self;
+    my $self = self;
 
 =head2 user_agent
 
 This method gets/sets the user-agent for the current scraper application instance.
 
-    init;
-    user_agent 'Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US; rv:1.9.2.8) Gecko/20100722 Firefox/3.6.8';
+    user_agent 'Mozilla/5.0 (Windows; U; Windows NT ...';
 
 =head2 var
 
@@ -886,16 +1044,9 @@ stash object.
 
     var age => 31;
     print var->{age};
-    # 31
     
     my @array = (1..20);
     var integers => @array;
-    
-    var->{foo}->{bar} = 'baz';
-    
-    # stash variable nesting ** depreciated ** not recommended **
-    var 'user/profile/name' => 'Mr. Foobar';
-    print var->{user}->{profile}->{name};
 
 =head2 random_ua
 
@@ -904,17 +1055,15 @@ user-agent header in your request is how an inquiring application might determin
 the browser and environment making the request. The first argument should be the
 name of the web browser, supported web browsers are any, chrome, ie or explorer,
 opera, safari, and firfox. Obviously using the keyword `any` will select from
-any available browser. The second argument which is optional should be the name
+any available browsers. The second argument which is optional should be the name
 of the desired operating system, supported operating systems are windows,
 macintosh, and linux. 
 
-    init;
     user_agent random_ua;
     # same as random_ua 'any';
 
 e.g. for a Linux-specific Google Chrome user-agent use the following...
 
-    init;
     user_agent random_ua 'chrome', 'linux';
 
 =head2 form
@@ -922,8 +1071,6 @@ e.g. for a Linux-specific Google Chrome user-agent use the following...
 The form method is a shortcut to the WWW::Mechanize submit_form method. It take
 the exact same arguments, yada, yada.
 
-    init;
-    get $requested_login_url;
     form fields => {
         username => 'mrmagoo',
         password => 'foobarbaz'
@@ -966,41 +1113,48 @@ HTTP::Response object. Alternatively the post object can be used traditionally
             post_form_id        => $post_formid,
             fb_dtsg             => 'u9MeI',
             post_form_id_source => 'AsyncRequest'
-        };
+    };
 
 Note! Our prettier version of the post method uses a content-type of
 application/x-www-form-urlencoded by default, to use multipart/form-data,
 please use the traditional style, sorry.
 
+=head2 param
+
+The param method is used to retrieve querystring parameters from the current request.
+This includes any parameters defined using the match() method. This method is never
+used to set parameters.
+
+    my $url = 'http://search.cpan.org/search?query=Scrappy&mode=all';
+    get $url;
+    
+    print param('query');
+    # Scrappy
+
 =head2 grab
 
-The grab method is a shortcut to the Web::Scraper process method. It take
-the exact same arguments with a little bit of our own added magic, namely you
-can grab and return single-selections and even specify the return values, by
-default the return value of a single-selection is TEXT. Note! Use a hashref
-mapping to return a list of results, this may change in the future.
+The grab method takes XPATH or CSS3 selectors and returns corresponding elements,
+it is a shortcut to the Web::Scraper process method. It take the exact same
+arguments with a little bit of our own added magic, namely you can grab and
+return a single element and specify whether to return TEXT, HTML or and @attribute.
+By default the return value of a single-element is TEXT. Whenever you specify a
+hashref mapping of attributes to grab, the results are returned as an arrayref,
+this may change in the future.
 
-    init;
-    get $requested_url;
     grab '#profile li a'; # return the inner text of the first encounter
     grab '#profile li a', '@href'; # specifically returning href attribute of the first encounter
     
-    # meaning you can do cool stuff like...
-    var user_name => grab '#profile li a';
-    
     # the traditional use is to provide a selector and mappings/return values e.g.
     grab '#profile li a', { name => 'TEXT', link => '@href' };
     
     # feeling lazy, let Scrappy auto-discover the attributes for you
-    grab '#profile li a', ':all';
+    grab '#profile li a', ':all'; # returns an arrayref if more than one element is found
     
-    # Note! using { mappings } or ':all' will return an arrayref and should be used when
-    # trying to select multiple records.
-    # Also Note! elements are returned as objects with accessors making it possible
+    # Note! elements are returned as objects with accessors making it possible
     # to do the following....
     
-    my $label = grab '#profile li a', ':all';
-    print $label->href;
+    my $link = grab '#profile li a:first', ':all';
+    print $link->href;
     
     grab 'a'; # returns inner text of the first match
     grab 'a', 'html'; # returns inner html of the first match
@@ -1009,31 +1163,15 @@ mapping to return a list of results, this may change in the future.
     grab 'a', ':all'; # returns an arrayref with all attributes including text, and html
     grab 'a', { key => 'attr' }; # returns an arrayref with the specified attributes
 
-=head2 zoom
+Zoom in on specific chunks of html code or pass you own using the following method call:
 
-The zoom method is almost exactly the same as the Scrappy grab method except
-that you specify what data to scrape as opposed to the grab method that parses
-the entire page. This is more of a drill-down utility. Note! Use a hashref
-mapping to return a list of results, this may change in the future.
-
-    init;
-    get $requested_url;
-    
-    var items => grab '#find ul li', { id => '@id', content => 'HTML' };
-    
-    foreach my $el (list var->{items}) {
-        var->{$el->{id}}->{title} => zoom $el->{content}, '.title';
-    }
-    
-    # just a silly example but zoom has many very good uses
-    # it is more of a drill-down utility
+    grab 'element', ':all', $html_content;
 
 =head2 loaded
 
 The loaded method is a shortcut to the WWW::Mechanize success method. This
 method returns true/false based on whether the last request was successful.
 
-    init;
     get $requested_url;
     if (loaded) {
         grab ...
@@ -1044,7 +1182,6 @@ method returns true/false based on whether the last request was successful.
 The status method is a shortcut to the WWW::Mechanize status method. This
 method returns the 3-digit HTTP status code of the response.
 
-    init;
     get $requested_url;
     if (status == 200) {
         grab ...
@@ -1107,9 +1244,9 @@ returns the content of the current page.
 =head2 data
 
 The data method is a shortcut to the WWW::Mechanize content method. This method
-returns the content of the current page. Additionally this method when passed
-data, updates the content of the current page with that data and
-returns the modified content.
+returns the content of the current page exactly the same as the html function does.
+Additionally this method when passed data, updates the content of the current page
+with that data and returns the modified content.
 
 =head2 www
 
@@ -1186,7 +1323,7 @@ and note that there is a newline on the alst line of the file:
 
 =head2 config
 
-The config method is an alias to the Scrappy session method for readability.
+The config method is an alias to the Scrappy session method for brevity.
 
 =head2 cookies
 
@@ -1195,12 +1332,10 @@ cookie handler. This method returns an HTTP::Cookie object. Setting this as
 undefined using the _undef keyword will prevent cookies from being stored and
 subsequently read.
 
-    init;
     get $requested_url;
     my $cookies = cookies;
     
     # prevent cookie storage
-    init;
     cookies _undef;
 
 =head2 proxy
@@ -1210,11 +1345,9 @@ set the proxy for the next request to be tunneled through. Setting this as
 undefined using the _undef keyword will reset the scraper application instance
 so that all subsequent requests will not use a proxy.
 
-    init;
     proxy 'http', 'http://proxy.example.com:8000/';
     get $requested_url;
     
-    init;
     proxy 'http', 'ftp', 'http://proxy.example.com:8000/';
     get $requested_url;
     
@@ -1222,7 +1355,6 @@ so that all subsequent requests will not use a proxy.
     
     use Tiny::Try;
     
-    init;
     proxy 'http', 'ftp', 'http://proxy.example.com:8000/';
     
     try {
@@ -1230,7 +1362,7 @@ so that all subsequent requests will not use a proxy.
     };
 
 Note! When using a proxy to perform requests, be aware that if they fail your
-program will die unless you wrap yoru code in an eval statement or use a try/catch
+program will die unless you wrap your code in an eval statement or use a try/catch
 module. In the example above we use Tiny::Try to trap an errors that might occur
 when using a proxy.
 
@@ -1239,17 +1371,16 @@ when using a proxy.
 The pause method is an adaptation of the WWW::Mechanize::Sleep module. This method
 sets breaks between your requests in an attempt to simulate human interaction.
 
-    init;
     pause 20;
     
     get $request_1;
     get $request_2;
     get $request_3;
 
-The will be a break between each request made, get, post, request, etc., You can
-also specify a range to have the pause method select from at random...
+Given the above example, there will be a 20 sencond break between each request made,
+get, post, request, etc., You can also specify a range to have the pause method
+select from at random...
 
-    init;
     pause 5,20;
     
     get $request_1;
@@ -1266,41 +1397,6 @@ download one could obviously...
     download $requested_url, '/tmp';
     sleep pause();
 
-=head2 cursor
-
-The cursor method is used internally by the crawl method to determine what
-pages in the queue should be fetched next after the completion of the current
-fetch. This method returns the position of the cursor in the queue.
-
-=head2 queue
-
-The queue method is used to add valid URIs to the page fetching queue used by
-the crawl method internally, or to return the list of added URIs in the order
-received/input.
-
-    queue $new_url;
-    my @urls = queue;
-
-=head2 crawl
-
-The crawl method is designed to automatically and systematically crawl, spider,
-or fetch webpages and perform actions on selected elements on each page.
-
-    crawl $starting_url, {
-        'a' => sub {
-            # find all links and add them to the queue to be crawled
-            queue shift->href;
-        },
-        '/*' => sub {
-            # /* simply matches the root node, same as using 'body' in
-            # html page context, maybe do soemthing with shift->text or shift->html
-        },
-        'img' => sub {
-            # print all image URLs
-            print shift->src, "\n"
-        }
-    };
-
 =head2 history
 
 The history method returns a list of visted pages.
@@ -1319,6 +1415,7 @@ that require authentication and redirect if not authorized. This function
 return boolean, 1 if the current page doesn't match the requested page.
 
     get $url_to_dashboard;
+    
     if (denied) {
         # do login, again
     }
@@ -1326,9 +1423,106 @@ return boolean, 1 if the current page doesn't match the requested page.
         # resume ...
     }
 
+=head2 new
+
+The new method creates a new OO (object-oriented) Scrappy instance. It is worth mentioning that
+Scrappy can be used in both OO (object-oriented) and DSL (domain-specific) fashion.
+Both styles have advantages and drawbacks, we have both so that settles that.
+Please note that a Scrappy instance is created automatically on-the-fly for those
+using DSL syntax.
+
+    my $spidy = Scrappy->new;
+
+=head2 cursor
+
+The cursor method is used internally by the crawl method to determine what
+pages in the queue should be fetched next after the completion of the current
+fetch. This method returns the position of the cursor in the queue.
+
+=head2 queue
+
+The queue method is used to add valid URIs to the page fetching queue used by
+the crawl method internally, or to return the list of added URIs in the order
+received/input.
+
+    queue $new_url;
+    my @urls = queue;
+
+=head2 match
+
+The match method checks the passed-in URL (or URL of the current page if left empty) the URL pattern
+(route) defined. If URL is a match, it will return the parameters of that match much in the same way
+a modern web application framework processes URL routes. 
+
+    my $url = 'http://somesite.com/tags/awesomeness';
+    ...
+    
+    # match against the current page
+    if (match '/tags/:tag') {
+        print param('tag');
+        # prints awesomeness
+    }
+    
+    .. or ..
+    
+    # match against the passed url
+    my $this = match '/tags/:tag', $url, {
+        host => 'somesite.com'
+    };
+    
+    if ($this) {
+        print "This is the ", $this->{tag}, " page";
+        # prints this is the awesomeness page
+    }
+
+=head2 crawl
+
+The crawl method is designed to automatically and systematically crawl, spider,
+or fetch webpages and perform actions on selected elements on each page. This
+method will start by GETting the initial URL passed, it then iterates over each
+selector executing the corresponding routine for each matched element.
+
+    crawl $starting_url, {
+        'a' => sub {
+            # find all links and add them to the queue to be crawled
+            queue shift->href;
+        },
+        '/*' => sub {
+            # /* simply matches the root node, same as using 'body' in
+            # html page context, maybe do soemthing with shift->text or shift->html
+        },
+        'img' => sub {
+            # print all image URLs
+            print shift->src, "\n"
+        }
+    };
+
+Lets take it a step further and as opposed to matching elements on every page we encounter,
+lets perform actions on elements that appear on specific types of pages. We do this by utilizing
+URL pattern matching (also known as URL routing in web application framework context).
+
+    crawl 'http://search.cpan.org/recent', {
+        'a' => sub {
+            my $link = shift;
+            queue $link->href if
+            match '/~:author/:dist/', $link->href;
+        },
+        '/~:author/:dist/' => {
+            'body', sub {
+                print "Howdy, I'm looking at " . param('author') . "\n";
+            },
+        }
+    };
+
+Just to recap, the above example starts crawling at http://search.cpan.org/recent, for the first page
+and every page crawled thereafter, Scrappy will look for the 'a' tag (links) and place them in the queue
+only if they match the defined URL pattern. Also, you'll notice the slightly different structure for the
+second action, which denotes a page action. This basically reads, if the current page matches this URL pattern
+apply the corresponding element actions.
+
 =head1 AUTHOR
 
-Al Newkirk <awncorp@cpan.org>
+  Al Newkirk <awncorp@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
