@@ -1,24 +1,24 @@
-# ABSTRACT: All Powerful Web Harvester, Spider, Scraper
+# ABSTRACT: A Magical Scrappy Namespace Where Wild Things Are
 package Scrappy::Doo;
 BEGIN {
-  $Scrappy::Doo::VERSION = '0.593';
+  $Scrappy::Doo::VERSION = '0.6';
 }
-use base 'Scrappy';
+use Scrappy qw/:syntax/;
 1;
 __END__
 =pod
 
 =head1 NAME
 
-Scrappy::Doo - All Powerful Web Harvester, Spider, Scraper
+Scrappy::Doo - A Magical Scrappy Namespace Where Wild Things Are
 
 =head1 VERSION
 
-version 0.593
+version 0.6
 
 =head1 AUTHOR
 
-Al Newkirk <awncorp@cpan.org>
+  Al Newkirk <awncorp@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
